@@ -27,10 +27,10 @@ impl SetAsync for ItemFn {
 
 fn trace_argument_code_string(arg_name: &str) -> String {
     let arg_name = ident(arg_name);
-    let statment: Stmt = parse_quote! {
+    let statement: Stmt = parse_quote! {
         println!("{} = {:?}", stringify!(#arg_name) ,#arg_name);
     };
-    statment.display_code()
+    statement.display_code()
 }
 
 mod single_test_should {
@@ -80,6 +80,20 @@ mod single_test_should {
         assert_eq!(inner_fn_impl.display_code(), input_fn.block.display_code());
     }
 
+    #[test]
+    fn not_remove_lifetimes() {
+        let input_fn: ItemFn = r#"
+                pub fn test<'a, 'b, 'c: 'a + 'b>(a: A<'a>, b: A<'b>, c: A<'c>) -> A<'c>
+                {
+                }
+                "#
+        .ast();
+
+        let result: ItemFn = single(input_fn.clone(), Default::default()).ast();
+
+        assert_eq!(3, result.sig.generics.lifetimes().count());
+    }
+
     #[rstest]
     fn not_copy_any_attributes(
         #[values(
@@ -135,8 +149,8 @@ mod single_test_should {
         let input_fn: ItemFn = r#"fn test(a: i32, b:i32, c:i32) {} "#.ast();
         let mut info: RsTestInfo = Default::default();
         info.arguments.set_global_await(true);
-        info.arguments.add_future(ident("a"));
-        info.arguments.add_future(ident("b"));
+        info.arguments.add_future(pat("a"));
+        info.arguments.add_future(pat("b"));
 
         let item_fn: ItemFn = single(input_fn.clone(), info).ast();
 
@@ -158,8 +172,8 @@ mod single_test_should {
     fn use_selective_await() {
         let input_fn: ItemFn = r#"fn test(a: i32, b:i32, c:i32) {} "#.ast();
         let mut info: RsTestInfo = Default::default();
-        info.arguments.set_future(ident("a"), FutureArg::Define);
-        info.arguments.set_future(ident("b"), FutureArg::Await);
+        info.arguments.set_future(pat("a"), FutureArg::Define);
+        info.arguments.set_future(pat("b"), FutureArg::Await);
 
         let item_fn: ItemFn = single(input_fn.clone(), info).ast();
 
@@ -177,6 +191,29 @@ mod single_test_should {
         );
     }
 
+    #[test]
+    fn use_ref_if_any() {
+        let input_fn: ItemFn = r#"fn test(a: i32, b:i32, c:i32) {} "#.ast();
+        let mut info: RsTestInfo = Default::default();
+        info.arguments.set_by_ref(pat("a"));
+        info.arguments.set_by_ref(pat("c"));
+
+        let item_fn: ItemFn = single(input_fn.clone(), info).ast();
+
+        assert_in!(
+            item_fn.block.stmts.last().display_code(),
+            ref_argument_code_string("a")
+        );
+        assert_not_in!(
+            item_fn.block.stmts.last().display_code(),
+            ref_argument_code_string("b")
+        );
+        assert_in!(
+            item_fn.block.stmts.last().display_code(),
+            ref_argument_code_string("c")
+        );
+    }
+
     #[test]
     fn trace_arguments_values() {
         let input_fn: ItemFn = r#"#[trace]fn test(s: String, a:i32) {} "#.ast();
@@ -200,7 +237,7 @@ mod single_test_should {
                 .ast();
 
         let mut attributes = RsTestAttributes::default();
-        attributes.add_notraces(vec![ident("b_no_trace"), ident("c_no_trace")]);
+        attributes.add_notraces(vec![pat("b_no_trace"), pat("c_no_trace")]);
 
         let item_fn: ItemFn = single(
             input_fn.clone(),
@@ -250,8 +287,9 @@ mod single_test_should {
 
         let result: ItemFn = single(input_fn.clone(), Default::default()).ast();
 
-        assert_eq!(result.attrs[0], test_attribute);
-        assert_eq!(&result.attrs[1..], attributes.as_slice());
+        let (generated_attribute, old_attributes) = result.attrs.split_last().unwrap();
+        assert_eq!(old_attributes, attributes.as_slice());
+        assert_eq!(generated_attribute, &test_attribute);
     }
 
     #[rstest]
@@ -276,8 +314,8 @@ mod single_test_should {
         .ast();
 
         let mut arguments = ArgumentsInfo::default();
-        arguments.add_future(ident("async_ref_u32"));
-        arguments.add_future(ident("async_u32"));
+        arguments.add_future(pat("async_ref_u32"));
+        arguments.add_future(pat("async_u32"));
 
         let info = RsTestInfo {
             arguments,
@@ -289,8 +327,8 @@ mod single_test_should {
 
         let expected = parse_str::<syn::ItemFn>(
             r#"async fn test<'_async_ref_u32>(
-                        async_ref_u32: impl std::future::Future<Output = &'_async_ref_u32 u32>, 
-                        async_u32: impl std::future::Future<Output = u32>, 
+                        async_ref_u32: impl core::future::Future<Output = &'_async_ref_u32 u32>, 
+                        async_u32: impl core::future::Future<Output = u32>, 
                         simple: u32
                     )
                     { }
@@ -302,6 +340,7 @@ mod single_test_should {
     }
 }
 
+#[derive(Debug)]
 struct TestsGroup {
     requested_test: ItemFn,
     module: ItemMod,
@@ -486,14 +525,14 @@ mod cases_should {
 
     use crate::parse::{
         arguments::{ArgumentsInfo, FutureArg},
-        rstest::RsTestItem,
+        rstest::{RsTestData, RsTestItem},
     };
 
     use super::{assert_eq, *};
 
     fn into_rstest_data(item_fn: &ItemFn) -> RsTestData {
         RsTestData {
-            items: fn_args_idents(item_fn)
+            items: fn_args_pats(item_fn)
                 .cloned()
                 .map(RsTestItem::CaseArgName)
                 .collect(),
@@ -534,8 +573,8 @@ mod cases_should {
             (self.item_fn, self.info)
         }
 
-        fn add_notrace(mut self, idents: Vec<Ident>) -> Self {
-            self.info.attributes.add_notraces(idents);
+        fn add_notrace(mut self, pats: Vec<Pat>) -> Self {
+            self.info.attributes.add_notraces(pats);
             self
         }
     }
@@ -610,7 +649,7 @@ mod cases_should {
         assert!(tests.len() > 0);
 
         for t in tests {
-            assert_eq!(item_fn.attrs, &t.attrs[1..]);
+            assert_eq!(item_fn.attrs, &t.attrs[..t.attrs.len() - 1]);
         }
     }
 
@@ -631,7 +670,8 @@ mod cases_should {
 
         let tokens = parametrize(item_fn, info);
 
-        let test_attrs = &TestsGroup::from(tokens).get_all_tests()[0].attrs[1..];
+        let tests = TestsGroup::from(tokens).get_all_tests();
+        let test_attrs = tests[0].attrs.split_last().unwrap().1;
 
         let l = given_attrs.len();
 
@@ -848,9 +888,10 @@ mod cases_should {
         let tokens = parametrize(item_fn, info);
 
         let tests = TestsGroup::from(tokens).get_all_tests();
+        let (generated_attribute, old_attributes) = tests[0].attrs.split_last().unwrap();
 
-        assert_eq!(tests[0].attrs[0], test_attribute);
-        assert_eq!(&tests[0].attrs[1..], attributes.as_slice());
+        assert_eq!(old_attributes, attributes.as_slice());
+        assert_eq!(generated_attribute, &test_attribute);
     }
 
     #[test]
@@ -861,8 +902,8 @@ mod cases_should {
         .take();
 
         let mut arguments = ArgumentsInfo::default();
-        arguments.add_future(ident("async_ref_u32"));
-        arguments.add_future(ident("async_u32"));
+        arguments.add_future(pat("async_ref_u32"));
+        arguments.add_future(pat("async_u32"));
 
         info.arguments = arguments;
 
@@ -871,8 +912,8 @@ mod cases_should {
 
         let expected = parse_str::<syn::ItemFn>(
             r#"async fn test<'_async_ref_u32>(
-                        async_ref_u32: impl std::future::Future<Output = &'_async_ref_u32 u32>, 
-                        async_u32: impl std::future::Future<Output = u32>, 
+                        async_ref_u32: impl core::future::Future<Output = &'_async_ref_u32 u32>, 
+                        async_u32: impl core::future::Future<Output = u32>, 
                         simple: u32
                     )
                     { }
@@ -928,7 +969,7 @@ mod cases_should {
             TestCaseBuilder::from(r#"#[trace] fn test(a_trace_me: i32, b_no_trace_me: i32, c_no_trace_me: i32, d_trace_me: i32) {}"#)
                 .push_case(TestCase::from_iter(vec!["1", "2", "1", "2"]))
                 .push_case(TestCase::from_iter(vec!["3", "4", "3", "4"]))
-                .add_notrace(to_idents!(["b_no_trace_me", "c_no_trace_me"]))
+                .add_notrace(to_pats!(["b_no_trace_me", "c_no_trace_me"]))
                 .take();
 
         let tokens = parametrize(item_fn, info);
@@ -958,7 +999,7 @@ mod cases_should {
             TestCaseBuilder::from(r#"fn test(a_no_trace_me: i32, b_trace_me: i32) {}"#)
                 .push_case(TestCase::from_iter(vec!["1", "2"]))
                 .push_case(TestCase::from_iter(vec!["3", "4"]).with_attrs(attrs("#[trace]")))
-                .add_notrace(to_idents!(["a_no_trace_me"]))
+                .add_notrace(to_pats!(["a_no_trace_me"]))
                 .take();
 
         let tokens = parametrize(item_fn, info);
@@ -986,8 +1027,8 @@ mod cases_should {
             .push_case(TestCase::from_iter(vec!["1", "2", "3"]))
             .take();
         info.arguments.set_global_await(true);
-        info.arguments.add_future(ident("a"));
-        info.arguments.add_future(ident("b"));
+        info.arguments.add_future(pat("a"));
+        info.arguments.add_future(pat("b"));
 
         let tokens = parametrize(item_fn, info);
 
@@ -1006,8 +1047,8 @@ mod cases_should {
             .push_case(TestCase::from_iter(vec!["1", "2", "3"]))
             .push_case(TestCase::from_iter(vec!["1", "2", "3"]))
             .take();
-        info.arguments.set_future(ident("a"), FutureArg::Define);
-        info.arguments.set_future(ident("b"), FutureArg::Await);
+        info.arguments.set_future(pat("a"), FutureArg::Define);
+        info.arguments.set_future(pat("b"), FutureArg::Await);
 
         let tokens = parametrize(item_fn, info);
 
@@ -1019,12 +1060,54 @@ mod cases_should {
         assert_in!(code, await_argument_code_string("b"));
         assert_not_in!(code, await_argument_code_string("c"));
     }
+
+    #[test]
+    fn render_context() {
+        let (item_fn, mut info) =
+            TestCaseBuilder::from(r#"fn test_with_context(ctx: Context, a: u32) {}"#)
+                .push_case(TestCase {
+                    args: vec![expr("1")],
+                    attrs: Default::default(),
+                    description: Some(ident("my_description")),
+                })
+                .push_case(TestCase {
+                    args: vec![expr("2")],
+                    attrs: Default::default(),
+                    description: Some(ident("other_description")),
+                })
+                .take();
+        info.arguments.add_context(pat("ctx"));
+
+        let tokens = parametrize(item_fn, info);
+
+        let tests = TestsGroup::from(tokens);
+
+        fn code(tests: &TestsGroup, id: usize) -> String {
+            tests.module.get_all_tests()[id].block.display_code()
+        }
+
+        assert_in!(
+            code(&tests, 0),
+            r#"let ctx = Context::new(module_path!(), "test_with_context", Some("my_description"), Some(0usize));"#
+                .ast::<Stmt>()
+                .display_code()
+        );
+        assert_in!(
+            code(&tests, 1),
+            r#"let ctx = Context::new(module_path!(), "test_with_context", Some("other_description"), Some(1usize));"#
+                .ast::<Stmt>()
+                .display_code()
+        );
+    }
 }
 
 mod matrix_cases_should {
     use rstest_test::{assert_in, assert_not_in};
 
-    use crate::parse::arguments::{ArgumentsInfo, FutureArg};
+    use crate::parse::{
+        arguments::{ArgumentsInfo, FutureArg},
+        rstest::RsTestData,
+    };
 
     /// Should test matrix tests render without take in account MatrixInfo to RsTestInfo
     /// transformation
@@ -1032,7 +1115,7 @@ mod matrix_cases_should {
 
     fn into_rstest_data(item_fn: &ItemFn) -> RsTestData {
         RsTestData {
-            items: fn_args_idents(item_fn)
+            items: fn_args_pats(item_fn)
                 .cloned()
                 .map(|it| {
                     ValueList {
@@ -1133,8 +1216,8 @@ mod matrix_cases_should {
         assert!(tests.len() > 0);
 
         for t in tests {
-            let end = t.attrs.len() - 1;
-            assert_eq!(item_fn.attrs, &t.attrs[1..end]);
+            let end = t.attrs.len() - 2;
+            assert_eq!(item_fn.attrs, &t.attrs[0..end]);
         }
     }
 
@@ -1249,12 +1332,12 @@ mod matrix_cases_should {
         assert!(tests.len() > 0);
 
         for test in tests {
-            let filterd: Vec<_> = test
+            let filtered: Vec<_> = test
                 .attrs
                 .into_iter()
                 .filter(|a| !filter.contains(a))
                 .collect();
-            assert_eq!(attributes, filterd);
+            assert_eq!(attributes, filtered);
         }
     }
 
@@ -1290,8 +1373,8 @@ mod matrix_cases_should {
         assert!(tests.len() > 0);
 
         for test in tests {
-            assert_eq!(test.attrs[0], test_attribute);
-            assert_eq!(&test.attrs[1..test.attrs.len() - 1], attributes.as_slice());
+            assert_eq!(&test.attrs[..test.attrs.len() - 2], attributes.as_slice());
+            assert_eq!(test.attrs[test.attrs.len() - 1], test_attribute);
         }
     }
 
@@ -1300,8 +1383,8 @@ mod matrix_cases_should {
         let item_fn = r#"async fn test(async_ref_u32: &u32, async_u32: u32,simple: u32) { }"#.ast();
 
         let mut arguments = ArgumentsInfo::default();
-        arguments.add_future(ident("async_ref_u32"));
-        arguments.add_future(ident("async_u32"));
+        arguments.add_future(pat("async_ref_u32"));
+        arguments.add_future(pat("async_u32"));
 
         let info = RsTestInfo {
             arguments,
@@ -1314,8 +1397,8 @@ mod matrix_cases_should {
 
         let expected = parse_str::<syn::ItemFn>(
             r#"async fn test<'_async_ref_u32>(
-                        async_ref_u32: impl std::future::Future<Output = &'_async_ref_u32 u32>, 
-                        async_u32: impl std::future::Future<Output = u32>, 
+                        async_ref_u32: impl core::future::Future<Output = &'_async_ref_u32 u32>, 
+                        async_u32: impl core::future::Future<Output = u32>, 
                         simple: u32
                     )
                     { }
@@ -1354,8 +1437,8 @@ mod matrix_cases_should {
         assert!(tests.len() > 0);
 
         for test in tests {
-            assert_eq!(test.attrs.last().unwrap(), non_snake_case);
-            assert_eq!(&test.attrs[1..test.attrs.len() - 1], attributes.as_slice());
+            assert_eq!(&test.attrs[test.attrs.len() - 2], non_snake_case);
+            assert_eq!(&test.attrs[..test.attrs.len() - 2], attributes.as_slice());
         }
     }
 
@@ -1418,7 +1501,7 @@ mod matrix_cases_should {
             .into(),
         };
         let mut attributes: RsTestAttributes = Default::default();
-        attributes.add_notraces(vec![ident("b_no_trace_me"), ident("c_no_trace_me")]);
+        attributes.add_notraces(vec![pat("b_no_trace_me"), pat("c_no_trace_me")]);
         let item_fn: ItemFn = r#"#[trace] fn test(a_trace_me: u32, b_no_trace_me: u32, c_no_trace_me: u32, d_trace_me: u32) {}"#.ast();
 
         let tokens = matrix(
@@ -1466,8 +1549,8 @@ mod matrix_cases_should {
             arguments: Default::default(),
         };
         info.arguments.set_global_await(true);
-        info.arguments.add_future(ident("a"));
-        info.arguments.add_future(ident("b"));
+        info.arguments.add_future(pat("a"));
+        info.arguments.add_future(pat("b"));
 
         let tokens = matrix(item_fn, info);
 
@@ -1497,8 +1580,8 @@ mod matrix_cases_should {
             arguments: Default::default(),
         };
 
-        info.arguments.set_future(ident("a"), FutureArg::Define);
-        info.arguments.set_future(ident("b"), FutureArg::Await);
+        info.arguments.set_future(pat("a"), FutureArg::Define);
+        info.arguments.set_future(pat("b"), FutureArg::Await);
 
         let tokens = matrix(item_fn, info);
 
@@ -1694,9 +1777,45 @@ mod matrix_cases_should {
         assert_in!(functions[0], "third_1");
         assert_in!(functions[1], "third_2");
     }
+
+    #[test]
+    fn render_context() {
+        let item_fn: ItemFn =
+            r#"fn matrix_with_context(ctx: Context, first: u32, second: u32) { println!("user code") }"#.ast();
+        let values = vec!["1".to_string(), "2".to_string()];
+        let mut info = RsTestInfo {
+            data: RsTestData {
+                items: vec![
+                    values_list("first", values.as_ref()).into(),
+                    values_list("second", values.as_ref()).into(),
+                ],
+            },
+            ..Default::default()
+        };
+        info.arguments.add_context(pat("ctx"));
+
+        let tokens = matrix(item_fn.clone(), info);
+
+        let tests = TestsGroup::from(tokens);
+
+        fn code(tests: &TestsGroup, id: usize) -> String {
+            tests.module.get_all_tests()[id].block.display_code()
+        }
+
+        for t in 0..tests.module.get_all_tests().len() {
+            assert_in!(
+                code(&tests, t),
+                r#"let ctx = Context::new(module_path!(), "matrix_with_context", None, None);"#
+                    .ast::<Stmt>()
+                    .display_code()
+            );
+        }
+    }
 }
 
 mod complete_should {
+    use crate::parse::rstest::RsTestData;
+
     use super::{assert_eq, *};
 
     fn rendered_case(fn_name: &str) -> TestsGroup {
@@ -1810,11 +1929,11 @@ mod complete_should {
         let attrs = attrs("#[first]#[second(arg)]");
 
         for f in modules[0].get_all_tests() {
-            let end = f.attrs.len() - 1;
-            assert_eq!(attrs, &f.attrs[1..end]);
+            let end = f.attrs.len() - 2;
+            assert_eq!(attrs, &f.attrs[..end]);
         }
         for f in modules[1].get_all_tests() {
-            assert_eq!(attrs, &f.attrs[1..3]);
+            assert_eq!(attrs, &f.attrs[..2]);
         }
     }
     #[test]
@@ -1823,7 +1942,7 @@ mod complete_should {
         let attrs = attrs("#[third]#[forth(other)]");
 
         for f in modules[1].get_all_tests() {
-            assert_eq!(attrs, &f.attrs[3..5]);
+            assert_eq!(attrs, &f.attrs[2..4]);
         }
     }
 }
